@@ -13,6 +13,8 @@ import os
 import warnings
 from os import path
 
+from six import text_type
+
 from docutils import nodes
 from docutils.io import FileOutput
 from docutils.utils import new_document
@@ -21,7 +23,7 @@ from docutils.frontend import OptionParser
 from sphinx import package_dir, addnodes, highlighting
 from sphinx.deprecation import RemovedInSphinx17Warning
 from sphinx.config import string_classes, ENUM
-from sphinx.errors import SphinxError
+from sphinx.errors import SphinxError, ConfigError
 from sphinx.locale import _
 from sphinx.builders import Builder
 from sphinx.environment import NoUri
@@ -218,6 +220,12 @@ class LaTeXBuilder(Builder):
                 copy_asset_file(path.join(staticdirname, filename),
                                 self.outdir, context=context)
 
+        # use pre-1.6.x Makefile for make latexpdf on Windows
+        if os.name == 'nt':
+            staticdirname = path.join(package_dir, 'texinputs_win')
+            copy_asset_file(path.join(staticdirname, 'Makefile_t'),
+                            self.outdir, context=context)
+
         # copy additional files
         if self.config.latex_additional_files:
             logger.info(bold('copying additional files...'), nonl=1)
@@ -272,6 +280,23 @@ def validate_config_values(app):
                       "LaTeX markup since Sphinx 1.4.5 uses only prefixed macro names.",
                       RemovedInSphinx17Warning)
 
+    for document in app.config.latex_documents:
+        try:
+            text_type(document[2])
+        except UnicodeDecodeError:
+            raise ConfigError(
+                'Invalid latex_documents.title found (might contain non-ASCII chars. '
+                'Please use u"..." notation instead): %r' % (document,)
+            )
+
+        try:
+            text_type(document[3])
+        except UnicodeDecodeError:
+            raise ConfigError(
+                'Invalid latex_documents.author found (might contain non-ASCII chars. '
+                'Please use u"..." notation instead): %r' % (document,)
+            )
+
 
 def default_latex_engine(config):
     # type: (Config) -> unicode
