@@ -5,21 +5,29 @@
 
     Create a full-text search index for offline search.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 import re
+from os import path
 
 from six import iteritems, itervalues, text_type, string_types
 from six.moves import cPickle as pickle
+
 from docutils.nodes import raw, comment, title, Text, NodeVisitor, SkipNode
-from os import path
 
 import sphinx
 from sphinx.util import jsdump, rpartition
 from sphinx.util.pycompat import htmlescape
 from sphinx.search.jssplitter import splitter_code
 
+if False:
+    # For type annotation
+    from typing import Any, Dict, IO, Iterable, List, Tuple, Type, Set  # NOQA
+    from docutils import nodes  # NOQA
+    from sphinx.environment import BuildEnvironment  # NOQA
+
+
 class SearchLanguage(object):
     """
     This class is the base class for search natural language preprocessors.  If
@@ -42,10 +50,10 @@ class SearchLanguage(object):
        This class is used to preprocess search word which Sphinx HTML readers
        type, before searching index. Default implementation does nothing.
     """
-    lang = None
-    language_name = None
-    stopwords = set()
-    js_stemmer_rawcode = None
+    lang = None                 # type: unicode
+    language_name = None        # type: unicode
+    stopwords = set()           # type: Set[unicode]
+    js_stemmer_rawcode = None   # type: unicode
     js_stemmer_code = """
 /**
  * Dummy stemmer for languages without stemming rules.
@@ -55,28 +63,32 @@ var Stemmer = function() {
     return w;
   }
 }
-"""
+"""                             # type: unicode
 
     _word_re = re.compile(r'\w+(?u)')
 
     def __init__(self, options):
+        # type: (Dict) -> None
         self.options = options
         self.init(options)
 
     def init(self, options):
+        # type: (Dict) -> None
         """
         Initialize the class with the options the user has given.
         """
 
     def split(self, input):
+        # type: (unicode) -> List[unicode]
         """
         This method splits a sentence into words.  Default splitter splits input
         at white spaces, which should be enough for most languages except CJK
         languages.
         """
-        return self._word_re.findall(input)
+        return self._word_re.findall(input)  # type: ignore
 
     def stem(self, word):
+        # type: (unicode) -> unicode
         """
         This method implements stemming algorithm of the Python version.
 
@@ -90,6 +102,7 @@ var Stemmer = function() {
         return word
 
     def word_filter(self, word):
+        # type: (unicode) -> bool
         """
         Return true if the target word should be registered in the search index.
         This method is called after stemming.
@@ -107,6 +120,7 @@ from sphinx.search.en import SearchEnglish
 
 
 def parse_stop_word(source):
+    # type: (unicode) -> Set[unicode]
     """
     parse snowball style word list like this:
 
@@ -138,7 +152,7 @@ languages = {
     'sv': 'sphinx.search.sv.SearchSwedish',
     'tr': 'sphinx.search.tr.SearchTurkish',
     'zh': 'sphinx.search.zh.SearchChinese',
-}
+}   # type: Dict[unicode, Any]
 
 
 class _JavaScriptIndex(object):
@@ -151,9 +165,11 @@ class _JavaScriptIndex(object):
     SUFFIX = ')'
 
     def dumps(self, data):
+        # type: (Any) -> unicode
         return self.PREFIX + jsdump.dumps(data) + self.SUFFIX
 
     def loads(self, s):
+        # type: (str) -> Any
         data = s[len(self.PREFIX):-len(self.SUFFIX)]
         if not data or not s.startswith(self.PREFIX) or not \
            s.endswith(self.SUFFIX):
@@ -161,9 +177,11 @@ class _JavaScriptIndex(object):
         return jsdump.loads(data)
 
     def dump(self, data, f):
+        # type: (Any, IO) -> None
         f.write(self.dumps(data))
 
     def load(self, f):
+        # type: (IO) -> Any
         return self.loads(f.read())
 
 
@@ -176,12 +194,14 @@ class WordCollector(NodeVisitor):
     """
 
     def __init__(self, document, lang):
+        # type: (nodes.Node, SearchLanguage) -> None
         NodeVisitor.__init__(self, document)
-        self.found_words = []
-        self.found_title_words = []
+        self.found_words = []           # type: List[unicode]
+        self.found_title_words = []     # type: List[unicode]
         self.lang = lang
 
     def is_meta_keywords(self, node, nodetype):
+        # type: (nodes.Node, Type) -> bool
         if isinstance(node, sphinx.addnodes.meta) and node.get('name') == 'keywords':
             meta_lang = node.get('lang')
             if meta_lang is None:  # lang not specified
@@ -192,6 +212,7 @@ class WordCollector(NodeVisitor):
         return False
 
     def dispatch_visit(self, node):
+        # type: (nodes.Node) -> None
         nodetype = type(node)
         if issubclass(nodetype, comment):
             raise SkipNode
@@ -223,28 +244,29 @@ class IndexBuilder(object):
     formats = {
         'jsdump':   jsdump,
         'pickle':   pickle
-    }
+    }   # type: Dict[unicode, Any]
 
     def __init__(self, env, lang, options, scoring):
+        # type: (BuildEnvironment, unicode, Dict, unicode) -> None
         self.env = env
-        # docname -> title
-        self._titles = {}
-        # docname -> filename
-        self._filenames = {}
-        # stemmed word -> set(docname)
-        self._mapping = {}
-        # stemmed words in titles -> set(docname)
-        self._title_mapping = {}
-        # word -> stemmed word
-        self._stem_cache = {}
-        # objtype -> index
-        self._objtypes = {}
-        # objtype index -> (domain, type, objname (localized))
-        self._objnames = {}
-        # add language-specific SearchLanguage instance
-        lang_class = languages.get(lang)
+        self._titles = {}           # type: Dict[unicode, unicode]
+                                    # docname -> title
+        self._filenames = {}        # type: Dict[unicode, unicode]
+                                    # docname -> filename
+        self._mapping = {}          # type: Dict[unicode, Set[unicode]]
+                                    # stemmed word -> set(docname)
+        self._title_mapping = {}    # type: Dict[unicode, Set[unicode]]
+                                    # stemmed words in titles -> set(docname)
+        self._stem_cache = {}       # type: Dict[unicode, unicode]
+                                    # word -> stemmed word
+        self._objtypes = {}         # type: Dict[Tuple[unicode, unicode], int]
+                                    # objtype -> index
+        self._objnames = {}         # type: Dict[int, Tuple[unicode, unicode, unicode]]
+                                    # objtype index -> (domain, type, objname (localized))
+        lang_class = languages.get(lang)    # type: Type[SearchLanguage]
+                                            # add language-specific SearchLanguage instance
         if lang_class is None:
-            self.lang = SearchEnglish(options)
+            self.lang = SearchEnglish(options)  # type: SearchLanguage
         elif isinstance(lang_class, str):
             module, classname = lang_class.rsplit('.', 1)
             lang_class = getattr(__import__(module, None, None, [classname]),
@@ -262,6 +284,7 @@ class IndexBuilder(object):
         self.js_splitter_code = splitter_code
 
     def load(self, stream, format):
+        # type: (IO, Any) -> None
         """Reconstruct from frozen data."""
         if isinstance(format, string_types):
             format = self.formats[format]
@@ -275,6 +298,7 @@ class IndexBuilder(object):
         self._titles = dict(zip(index2fn, frozen['titles']))
 
         def load_terms(mapping):
+            # type: (Dict[unicode, Any]) -> Dict[unicode, Set[unicode]]
             rv = {}
             for k, v in iteritems(mapping):
                 if isinstance(v, int):
@@ -288,13 +312,15 @@ class IndexBuilder(object):
         # no need to load keywords/objtypes
 
     def dump(self, stream, format):
+        # type: (IO, Any) -> None
         """Dump the frozen index to a stream."""
         if isinstance(format, string_types):
             format = self.formats[format]
         format.dump(self.freeze(), stream)
 
     def get_objects(self, fn2index):
-        rv = {}
+        # type: (Dict[unicode, int]) -> Dict[unicode, Dict[unicode, Tuple[int, int, int, unicode]]]  # NOQA
+        rv = {}  # type: Dict[unicode, Dict[unicode, Tuple[int, int, int, unicode]]]
         otypes = self._objtypes
         onames = self._objnames
         for domainname, domain in sorted(iteritems(self.env.domains)):
@@ -321,7 +347,7 @@ class IndexBuilder(object):
                     else:
                         onames[typeindex] = (domainname, type, type)
                 if anchor == fullname:
-                    shortanchor = ''
+                    shortanchor = ''  # type: unicode
                 elif anchor == type + '-' + fullname:
                     shortanchor = '-'
                 else:
@@ -330,7 +356,8 @@ class IndexBuilder(object):
         return rv
 
     def get_terms(self, fn2index):
-        rvs = {}, {}
+        # type: (Dict) -> Tuple[Dict[unicode, List[unicode]], Dict[unicode, List[unicode]]]
+        rvs = {}, {}  # type: Tuple[Dict[unicode, List[unicode]], Dict[unicode, List[unicode]]]
         for rv, mapping in zip(rvs, (self._mapping, self._title_mapping)):
             for k, v in iteritems(mapping):
                 if len(v) == 1:
@@ -342,6 +369,7 @@ class IndexBuilder(object):
         return rvs
 
     def freeze(self):
+        # type: () -> Dict[unicode, Any]
         """Create a usable data structure for serializing."""
         docnames, titles = zip(*sorted(self._titles.items()))
         filenames = [self._filenames.get(docname) for docname in docnames]
@@ -357,9 +385,11 @@ class IndexBuilder(object):
                     titleterms=title_terms, envversion=self.env.version)
 
     def label(self):
+        # type: () -> unicode
         return "%s (code: %s)" % (self.lang.language_name, self.lang.lang)
 
     def prune(self, docnames):
+        # type: (Iterable[unicode]) -> None
         """Remove data for all docnames not in the list."""
         new_titles = {}
         new_filenames = {}
@@ -375,6 +405,7 @@ class IndexBuilder(object):
             wordnames.intersection_update(docnames)
 
     def feed(self, docname, filename, title, doctree):
+        # type: (unicode, unicode, unicode, nodes.Node) -> None
         """Feed a doctree to the index."""
         self._titles[docname] = title
         self._filenames[docname] = filename
@@ -384,6 +415,7 @@ class IndexBuilder(object):
 
         # memoize self.lang.stem
         def stem(word):
+            # type: (unicode) -> unicode
             try:
                 return self._stem_cache[word]
             except KeyError:
@@ -403,11 +435,12 @@ class IndexBuilder(object):
             # again, stemmer must not remove words from search index
             if not _filter(stemmed_word) and _filter(word):
                 stemmed_word = word
-            already_indexed = docname in self._title_mapping.get(stemmed_word, [])
+            already_indexed = docname in self._title_mapping.get(stemmed_word, set())
             if _filter(stemmed_word) and not already_indexed:
                 self._mapping.setdefault(stemmed_word, set()).add(docname)
 
     def context_for_searchtool(self):
+        # type: () -> Dict[unicode, Any]
         return dict(
             search_language_stemming_code = self.lang.js_stemmer_code,
             search_language_stop_words = jsdump.dumps(sorted(self.lang.stopwords)),
@@ -416,9 +449,12 @@ class IndexBuilder(object):
         )
 
     def get_js_stemmer_rawcode(self):
+        # type: () -> unicode
         if self.lang.js_stemmer_rawcode:
             return path.join(
                 sphinx.package_dir, 'search',
                 'non-minified-js',
                 self.lang.js_stemmer_rawcode
             )
+        else:
+            return None
